#!/usr/bin/python3
import argparse
from pathlib import Path, PurePath

from alembic import command
from alembic.config import Config

# version module run_migrations (<version ssa>.<version module>).
__version__ = "5.1.0"

alembic_config = Config()
alembic_config.set_main_option(
    "script_location", str(PurePath(Path(__file__).resolve().parent).joinpath("migrations"))
)


class RunMigrations:
    """
    Имплементируем применение миграций через alembic.

    NOTE: В будущем добавить передаваемые аргументы для гибкости и модификации работы.

    Attributes:
        alembic_config: instance alembic configuration.
        args: arguments from parser.
    """

    def __init__(self, arguments):
        self.args = arguments
        self.alembic_config = alembic_config

    def show_migrations(self):
        command.history(self.alembic_config)

    def update_migrations(self, revision="head"):
        """
        Апдейт миграций.

        Args:
            revision (str): hash or alias revision.
        """

        if self.args.last:
            revision = "head"
        if self.args.hash:
            revision = self.args.hash
        command.upgrade(self.alembic_config, revision)

    def downgrade_migrations(self, revision="-1"):
        """
        Даунгрейд миграций.

        Args:
            revision (str): hash or alias revision.
        """

        if self.args.last:
            revision = "-1"
        if self.args.hash:
            revision = self.args.hash
        command.downgrade(self.alembic_config, revision)

    def run(self):
        """Run migrations."""
        if self.args.run in ("u", "up") and not self.args.show:
            self.update_migrations()
        elif self.args.run in ("d", "down") and not self.args.show:
            self.downgrade_migrations()
        elif self.args.show:
            self.show_migrations()


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Applies migrations.", epilog="Measure seven times, cut once. (c) Russian proverb"
    )
    parser.add_argument(
        "-r",
        "--run",
        help="run migrations",
        choices=["up", "u", "d", "down"],
        default="up",
    )
    parser.add_argument(
        "-l",
        "--last",
        action="store_true",
        help="during upgrade migrations to head. during downgrade back to previous migration",
    )
    parser.add_argument("--hash", help="hash migration")
    parser.add_argument("--show", action="store_true", help="show changeset scripts in chronological order")
    parser.add_argument("--version", action="version", version=__version__)

    args = parser.parse_args()
else:
    args = type(
        "args", (), {"run": "up", "last": False, "show": False, "hash": None, "version": __version__}
    )

RunMigrations(args).run()
